{% extends 'base.html' %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}

{% block content %}

  <h2>{% trans "404" %}</h2>

  <p>
    {% blocktrans trimmed %}
      Requested page {{ request_path }} was not found.
    {% endblocktrans %}
  </p>

  <p>
    {% blocktrans trimmed %}
      If you believe this missing page should exist, please file a bug at the
      FreedomBox Service (Plinth) project <a
      href="https://salsa.debian.org/freedombox-team/plinth/issues">issue
      tracker</a>.
    {% endblocktrans %}
  </p>

{% endblock %}
