{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}
  <h2>{{ title }}</h2>

  <p>{% trans "Roll back the system to this snapshot?" %}</p>

  <p>
    {% blocktrans trimmed %}
      A new snapshot with the current state of the file system will be
      automatically created. You will be able to undo a rollback by
      reverting to the newly created snapshot.
    {% endblocktrans %}
  </p>

  <div class="row">
    <div class="col-lg-12">
      <table class="table table-bordered table-condensed table-striped">
        <thead>
          <th>{% trans "Number" %}</th>
          <th>{% trans "Date" %}</th>
          <th>{% trans "Description" %}</th>
        </thead>
        <tbody>
          <tr>
            <td>{{ snapshot.number }}</td>
            <td>{{ snapshot.date }}</td>
            <td>{{ snapshot.description }}</td>
          </tr>
        </tbody>
      </table>
    </div>
  </div>

  <p>
    <form class="form" method="post">
      {% csrf_token %}

      <input type="submit" class="btn btn-primary"
             value="{% blocktrans trimmed with number=snapshot.number %}
                      Rollback to Snapshot #{{ number }}
                    {% endblocktrans %}"/>
    </form>
  </p>

{% endblock %}
