{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}
  {% block pagetitle %}
    <h2>{{ title }}</h2>
  {% endblock %}

  {% block description %}
    {% for paragraph in description %}
    <p>{{ paragraph|safe }}</p>
    {% endfor %}
  {% endblock %}

  <p>
    {% url 'config:index' as index_url %}
    {% if domain_names|length == 0 %}
      No domain(s) are set. You can setup your domain on the system at
      <a href="{{ index_url }}">Configure</a> page.
    {% endif %}
  </p>

  {% block status %}
  {% endblock %}

  {% block diagnostics %}
  {% endblock %}

  {% block configuration %}
    {% if domain_names|length > 0 %}
      <h3>Configuration</h3>
      <form class="form" method="post">
        {% csrf_token %}

        {{ form|bootstrap }}

        <input type="submit" class="btn btn-primary"
               value="{% trans "Update setup" %}"/>
      </form>
    {% endif %}
  {% endblock %}
{% endblock %}
