{% extends "base_firstboot.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block page_head %}
  <style type="text/css">
   .navbar-brand {
       display: none;
   }
  </style>
{% endblock %}

{% block content_row %}
  <p class="text-center">
    <img src="{% static 'theme/img/FreedomBox-logo-standard.png' %}"
         alt="{{ box_name }}" width="640"/>
  </p>

  <form class="form text-center" method="post">
    {% csrf_token %}

    <p>
      <input type="submit" class="btn btn-lg btn-primary"
             value="{% trans "Start Setup" %}"/>
    </p>
  </form>

  <p class="text-center" style="font-size: larger">
    {% blocktrans trimmed %}
      To complete the setup of your {{ box_name }}, please provide
      some basic information.
    {% endblocktrans %}
  </p>
{% endblock %}
