{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}

<div class="row">
  <div class="col-lg-12">
    <table class="table table-bordered table-striped diagnostics-results">
      <thead>
        <tr>
          <th class="diagnostic-test">{% trans "Test" %}</th>
          <th class="diagnostics-result">{% trans "Result" %}</th>
        </tr>
      </thead>
      <tbody>
        {% for test, result in results %}
          <tr>
            <td>{{ test }}</td>
            <td>
              {% if result == 'passed' %}
                <span class="label label-success">{{ result }}</span>
              {% elif result == 'failed' %}
                <span class="label label-danger">{{ result }}</span>
              {% elif result == 'error' %}
                <span class="label label-warning">{{ result }}</span>
              {% else %}
                {{ result }}
              {% endif %}
            </td>
          </tr>
        {% endfor %}
      </tbody>
    </table>
  </div>
</div>
