{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block page_head %}
  <style type="text/css">
    div.checkbox .help-block {
      display: inline-block;
      margin: 0px 10px;
    }
    input.btn {
      margin: 10px 15px;
    }
  </style>
{% endblock %}

{% block content %}

  <div class="alert alert-warning" role="alert">

    {% trans "<b>Warning:</b><br/>" %}

    <p>
      {% blocktrans trimmed %}
        Published services are accessible and attackable from the evil
        Internet.
      {% endblocktrans %}
    </p>

  </div>

  <form class="form predefined" method="post">
    {{ form.http|bootstrap_horizontal:'col-lg-0' }}
    {{ form.https|bootstrap_horizontal:'col-lg-0' }}
    {{ form.ssh|bootstrap_horizontal:'col-lg-0' }}
    {% csrf_token %}

    <input type="submit" class="btn btn-primary"
           value="{% trans "Save Services" %}"/>
  </form>

{% endblock %}

