{% extends 'base.html' %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}

{% block submenu %}
  {% if submenu %}
    <div class="sidebar">
      {% include "submenu.html" with menu=submenu %}
    </div>
  {% endif %}
{% endblock %}

{% block content %}

  <h2>{% trans "Services and Applications" %}</h2>

  <p>
    {% blocktrans trimmed %}
      You can install and run various services and applications on
      your {{ box_name }}. Click on any app page link on the left to
      read a description of the application and choose to install it.
    {% endblocktrans %}
  </p>

  <p>
    {% blocktrans trimmed %}
      This box can be your photo sharing site, your instant messaging
      site, your social networking site, your news site.  Remember web
      portals?  We can be one of those too.  Many of the services you
      use on the web could soon be on site and under your control!
    {% endblocktrans %}
  </p>

{% endblock %}
