{% extends "base_firstboot.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block page_head %}
  <style type="text/css">
   a.navbar-brand {
       display: none;
   }
  </style>
  <meta http-equiv="refresh" content="3" />
{% endblock %}


{% block content_row %}

  {% if is_first_setup_running %}
    <div class="alert alert-warning" role="alert">
      {% blocktrans trimmed %}
        Please wait for {{ box_name }} to finish installation.
        You can start using your {{ box_name }} once it is done.
      {% endblocktrans %}
    </div>
  {% endif %}

  <p class="text-center">
    <img src="{% static 'theme/img/FreedomBox-logo-standard.png' %}"
         alt="{{ box_name }}" width="640"/>
  </p>

{% endblock %}
