{% extends 'base.html' %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}

{% block content %}

  <h2>{% trans "Diagnostic Results" %}</h2>

  <h3>{% blocktrans %}Module: {{ module_name }}{% endblocktrans %}</h3>

  {% if results %}
    {% include "diagnostics_results.html" with results=results %}
  {% else %}
    <p>{% trans "This module does not support diagnostics" %}</p>
  {% endif %}

{% endblock %}
