{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}

{% block content %}

  <h3>{% trans "NAT type" %}</h3>

  <p>
    {% if nat_unchecked %}
      {% blocktrans trimmed %}
        NAT type was not detected yet. If you do not provide an "IP Check
        URL", we will not detect a NAT type.
      {% endblocktrans %}
    {% else %}
      {% if no_nat %}
        {% trans "Direct connection to the Internet." %}
      {% else %}
        {% blocktrans trimmed %}
          Behind NAT. This means that Dynamic DNS service will poll
          the "URL to look up public IP" for changes (the "URL to look
          up public IP" entry is needed for this, otherwise IP changes
          will not be detected). In case the WAN IP changes, it may
          take up to {{ timer }} minutes until your DNS entry is
          updated.
        {% endblocktrans %}
      {% endif %}
    {% endif %}
  </p>

  <h3>{% trans "Last update" %}</h3>

  <p>{{ last_update }}</p>

{% endblock %}
