{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}

  <form class="form" method="post">

    <div class="row">
      {% csrf_token %}
      <div class="col-sm-3 col-md-offset-3">
        {{ form|bootstrap }}

        <div class="text-center">
          <input type="submit" class="btn btn-primary"
                 value="{% trans "Login" %}" />
        </div>
     </div>
    </div>

    <input type="hidden" name="next" value="{{ next }}" />
  </form>

{% endblock %}

{% block page_js %}
  <script>
    $('#id_username').focus();
  </script>
{% endblock %}
