{% load i18n %}
{% load static %}
{% load plinth_extras %}

{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}
<!doctype html>
<!--[if lt IE 7 ]> <html class="ie ie6 no-js" lang="en"> <![endif]-->
<!--[if IE 7 ]>    <html class="ie ie7 no-js" lang="en"> <![endif]-->
<!--[if IE 8 ]>    <html class="ie ie8 no-js" lang="en"> <![endif]-->
<!--[if IE 9 ]>    <html class="ie ie9 no-js" lang="en"> <![endif]-->
<!--[if gt IE 9]><!--> <html class="no-js" lang="en"> <!--<![endif]-->
<!-- the "no-js" class is for Modernizr -->
<head>
  <meta charset="utf-8" />

  <!-- Always force latest IE rendering engine and Chrome Frame -->
  <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" />

  <!--  Mobile Viewport Fix http://j.mp/mobileviewport & http://davidbcalhoun.com/2010/viewport-metatag
  device-width : Occupy full width of the screen in its current orientation
  initial-scale = 1.0 retains dimensions instead of zooming out if page height > device height
  maximum-scale = 1.0 retains dimensions instead of zooming in if page width < device width
  -->
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />

  <!-- R2D2, you know better than to trust a strange computer! -->
  <meta name="robots" content="noindex, nofollow, noarchive" />
  <meta name="googlebot" content="noindex, nofollow, noarchive, nosnippet, noodp, noimageindex, notranslate" />
  <meta name="msnbot" content="noindex, nofollow, noarchive, noodp" />
  <meta name="slurp" content="noindex, nofollow, noarchive, noodp, noydir" />
  <meta name="description"
        content="{% blocktrans trimmed %}
                   Core funtionality and web interface for {{ box_name }}
                 {% endblocktrans %}" />
  {% block title %}
    <title>
      {% if title %} {{ title }} {% else %} {{ box_name }} {% endif %}
    </title>
  {% endblock %}

  <!-- Favicon. Works with IE >= 11 and other GUI browsers -->
  <link rel="icon" type="image/png"
        href="{% static 'theme/img/freedombox-logo-32px.png' %}"/>

  <!-- The is the icon for iOS's Web Clip. Size: 57x57 for older iPhones, 72x72 for iPads, 114x114 for iPhone4
  - To prevent iOS from applying its styles to the icon name it thusly: apple-touch-icon-precomposed.png
  - Transparency is not recommended (iOS will put a black BG behind the icon) -->
  <link rel="apple-touch-icon" sizes="57x57" href="{% static 'theme/img/apple-touch-icon-57px-precomposed.png' %}"/>
  <link rel="apple-touch-icon" sizes="72x72" href="{% static 'theme/img/apple-touch-icon-72px-precomposed.png' %}"/>
  <link rel="apple-touch-icon" sizes="114x114" href="{% static 'theme/img/apple-touch-icon-114px-precomposed.png' %}"/>

  <!-- Bootstrap base CSS -->
  <link rel="stylesheet" href="{% static '/javascript/bootstrap/css/bootstrap.min.css' %}"/>
  <link rel="stylesheet" href="{% static 'theme/css/plinth.css' %}"/>
  <link rel="stylesheet" href="{% static 'theme/css/responsive.css' %}"/>
  {% block app_head %}<!-- placeholder for app/module-specific head files -->{% endblock %}
  {% block page_head %}<!-- placeholder for page-specific head files -->{% endblock %}
</head>

<body>
<div id="wrapper">
<!--[if lt IE 7]><p class=chromeframe>Your browser is <em>ancient!</em> <a href="http://mozilla.org/firefox">Upgrade to a modern version of Firefox</a> to experience this site.</p><![endif]-->
  <div class="navbar navbar-fixed-top navbar-default" role="navigation">
    <div class="container">
      <div class="navbar-header">
        <button type="button" class="navbar-toggle" data-toggle="collapse"
                data-target=".navbar-collapse">
          <span class="sr-only">{% trans "Toggle navigation" %}</span>
          <span class="icon-bar"></span>
          <span class="icon-bar"></span>
          <span class="icon-bar"></span>
        </button>
        {% block mainmenu_left %}
          <a href="{% url 'index' %}"  class="navbar-brand"
             title="{{ box_name }}">
            <img src="{% static 'theme/img/freedombox-logo-32px.png' %}"
                 alt="{{ box_name }}" />
          </a>
        {% endblock %}
      </div>
      <div class="collapse navbar-collapse">
        <ul class="nav navbar-nav">
          {% if user.is_authenticated and user_is_admin %}
            <li>
              <a href="{% url 'index' %}" title="{% trans "Home" %}">
                {% trans "Home" %}
              </a>
            </li>
            <li>
              <a href="{% url 'apps' %}" title="{% trans "Apps" %}">
                <span class="glyphicon glyphicon-th"></span>
                {% trans "Apps" %}
              </a>
            </li>
            <li>
              <a href="{% url 'system' %}" title="{% trans "System" %}">
                <span class="glyphicon glyphicon-cog nav-icon"></span>
                {% trans "System" %}
              </a>
            </li>
          {% endif %}
        </ul>
        <ul class="nav navbar-nav navbar-right">
          {% block mainmenu_right %}

            {% if user.is_authenticated %}
              <li>
                <a href="{% url 'help:index' %}" title="{% trans "Help" %}">
                  <span class="glyphicon-question-sign glyphicon nav-icon"></span>
                </a>
              </li>
              <li class="dropdown">
                <a href="{% url 'users:edit' request.user.username %}"
                   class="dropdown-toggle" data-toggle="dropdown"
                   role="button" aria-expanded="false">
                  <i class="glyphicon glyphicon-user nav-icon"></i>
                  {{ request.user.username }}
                  <span class="caret"></span>
                </a>
                <ul class="dropdown-menu" role="menu">
                  <li>
                    <a href="{% url 'users:edit' request.user.username %}"
                       title="{% trans "Edit"%}">
                      {% trans "Edit" %}
                    </a>
                  </li>
                  <li>
                    <a href="{% url 'users:change_password' request.user.username %}"
                       title="{% trans "Change password" %}">
                      {% trans "Change password" %}
                    </a>
                  </li>
                  <li class="divider"></li>
                  {% if user_is_admin %}
                    <li>
                      <a href="{% url 'power:restart' %}"
                         title="{% trans "Restart"%}">
                        {% trans "Restart" %}
                      </a>
                    </li>
                    <li>
                      <a href="{% url 'power:shutdown' %}"
                         title="{% trans "Shut down" %}">
                        {% trans "Shut down" %}
                      </a>
                    </li>
                    <li class="divider"></li>
                  {% endif %}
                  <li>
                    <a href="{% url 'users:logout' %}"
                       title="{% trans "Log out" %}">
                      {% trans "Log out" %}
                    </a>
                  </li>
                </ul>
              </li>
            {% else %}
              <li>
                <a href="{% url 'language-selection' %}?next={{ request.path|iriencode }}"
                   title="{% trans "Select language" %}">
                  <i class="glyphicon glyphicon-globe nav-icon"></i>
                </a>
              </li>
              <li>
                  <a href="{% url 'users:login' %}" title="{% trans "Log in" %}">
                    <i class="glyphicon glyphicon-user nav-icon"></i>
                    {% trans "Log in" %}</a>
              </li>
            {% endif %}

            {% if user.is_authenticated %}
              <li id="logout-nojs">
                <a href="{% url 'users:logout' %}" title="{% trans "Log out" %}">
                  <i class="glyphicon glyphicon-remove-circle nav-icon"></i>
                  {% trans "Log out" %}</a>
              </li>
            {% endif %}
          {% endblock %}
        </ul>

      </div>
    </div>
  </div>

  <div id="container-wrapper" class="container">
    <div class="row">
      {% block content_row %}
        <div class="col-md-3">
          {% block submenu %}
            {% if submenu %}
              <div class="sidebar visible-lg-block">
                {% include "submenu.html" with menu=submenu %}
              </div><!--/.sidebar -->
            {% endif %}
          {% endblock %}

          {% block sidebar %}
            {# this sidebar should contain help texts but no menus #}
          {% endblock %}
        </div><!--/col-->

        <div class="col-md-9">
          {% block subsubmenu %}
            {% if subsubmenu %}
              {% show_subsubmenu subsubmenu %}
            {% endif %}
          {% endblock %}

          {% include 'messages.html' %}

          {% block content %}
            {# main content goes here #}
          {% endblock %}
        </div><!--/col-->

        <div class="col-md-12">
          {% block icons %}
            {# icons go here #}
          {% endblock %}
        </div><!--/col-->

      {% endblock %}
    </div><!--/row-->

    <div class="text-center">
      {% block center-info %}
        {# this center-info should contain help texts but no menus #}
      {% endblock %}
    </div>
  </div><!--/.fluid-container-->

  <footer>
    <hr>
    <div class="container">
      {% block footer_block %}
        <div class="license-info">
          <p>
            <a href="{% static 'jslicense.html' %}" data-jslicense="1">
              {% trans "JavaScript license information" %}</a>
          </p>
        </div>
      {% endblock %}
    </div>
  </footer><!--/.footer-->
</div><!--/#container-->
  <!-- JavaScript <script> tags are placed at the end of the document to speed up initial page loads-->
  <!-- Local link to system Modernizr (includes HTML5 Shiv) -->
  <script type="text/javascript" src="{% static '/javascript/modernizr/modernizr.min.js' %}"></script>
  <!-- Local link to system jQuery -->
  <script type="text/javascript" src="{% static '/javascript/jquery/jquery.min.js' %}"></script>
  <!-- Local link to system Bootstrap JS -->
  <script type="text/javascript" src="{% static '/javascript/bootstrap/js/bootstrap.min.js' %}"></script>

  {% block app_js %}<!-- placeholder for app-specific js files -->{% endblock %}
  {% block page_js %}<!-- placeholder for page-specific js files -->{% endblock %}
</body>
</html>
