{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{# Template to display/configure a Service, used by views.ServiceView #}

{% load bootstrap %}
{% load i18n %}

{% block content %}
  {% block pagetitle %}
    <h2>{{ service.name }}</h2>
  {% endblock %}

  {% block description %}
    {% for paragraph in description %}
      <p>{{ paragraph|safe }}</p>
    {% endfor %}
  {% endblock %}

  {% include "clients.html" with clients=clients %}

  {% block status %}
    {% if show_status_block %}
      <h3>{% trans "Status" %}</h3>
      <p class="running-status-parent">
        {% with service_name=service.name %}
          {% if service.is_running %}
            <span class="running-status active"></span>
            {% blocktrans trimmed %}
              Service <em>{{ service_name }}</em> is running.
            {% endblocktrans %}
          {% else %}
            <span class="running-status inactive"></span>
            {% blocktrans trimmed %}
              Service <em>{{ service_name }}</em> is not running.
            {% endblocktrans %}
          {% endif %}
        {% endwith %}
      </p>
    {% endif %}
  {% endblock %}

  {% block diagnostics %}
    {% if diagnostics_module_name %}
      {% include "diagnostics_button.html" with module=diagnostics_module_name enabled=service.is_enabled %}
    {% endif %}
  {% endblock %}

  {% block configuration %}
    <h3>{% trans "Configuration" %}</h3>

    <form class="form" method="post">
      {% csrf_token %}

      {{ form|bootstrap }}

      <input type="submit" class="btn btn-primary"
             value="{% trans "Update setup" %}"/>
    </form>
  {% endblock %}

{% endblock %}
