{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content_row %}

  <div class="col-lg-6 col-lg-offset-3">
    {% include 'messages.html' %}

    <h2>{% trans "Setup Complete!" %}</h2>

    <p>
      {% blocktrans trimmed %}
        Without any apps, your {{ box_name }} cannot do very much.
      {% endblocktrans %}
    </p>

    <div class="text-center">
      <a class="btn btn-lg btn-primary" href="{% url 'apps' %}">
        {% trans "Install Apps" %}</a>
    </div>

    <p>
      <p>
        {% url 'networks:index' as networks_url %}
        {% blocktrans trimmed %}
          You may want to check the <a href="{{ networks_url }}">network
          setup</a> and modify it if necessary.
        {% endblocktrans %}
      </p>
    </p>
  </div>

{% endblock %}
