{% extends 'base.html' %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}

{% block submenu %}
  {% if submenu %}
    <div class="sidebar">
      {% include "submenu.html" with menu=submenu %}
    </div>
  {% endif %}
{% endblock %}

{% block content %}

  <h2>{% trans "System Configuration" %}</h2>

  <p>
    {% blocktrans trimmed %}
      Here you can administrate the underlying system of your {{ box_name }}.
    {% endblocktrans %}
  </p>

  <p>
    {% blocktrans trimmed %}
      The options affect the {{ box_name }} at its most general level,
      so be careful!
    {% endblocktrans %}
  </p>
{% endblock %}
