{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}

<ul class="nav nav-pills nav-stacked">
  {% for item in menu.items %}

    {% if item.url in active_menu_urls %}
      <li class="active">
        <a href="{{ item.url }}" class="active">
    {% else %}
      <li>
        <a href="{{ item.url }}">
    {% endif %}
          <span class="{{ item.icon }} glyphicon"></span>
          {{ item.label }}
        </a>
        {% if item.items %}
          {% include "menu.html" with menu=item %}
        {% endif %}
      </li>

  {% endfor %}
</ul>
