{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}

{% block content %}
  <h2>{{ title }}</h2>

  {% for paragraph in description %}
    <p>{{ paragraph|safe }}</p>
  {% endfor %}

  <p>
    {% blocktrans trimmed %}
      If you are looking for a free dynamic DNS account, you may find
      a free GnuDIP service at <a href='http://gnudip.datasystems24.net'
      target='_blank'>gnudip.datasystems24.net</a> or you may find
      free update URL based services at
      <a href='http://freedns.afraid.org/' target='_blank'>
      freedns.afraid.org</a>.
    {% endblocktrans %}
  </p>

  <p>
    {% blocktrans trimmed %}
      If your {{ box_name }} is connected behind a NAT router, don't forget
      to add port forwarding for standard ports, including TCP port 80 (HTTP)
      and TCP port 443 (HTTPS).
    {% endblocktrans %}
  </p>

{% endblock %}
