{% extends 'base.html' %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load i18n %}
{% load static %}

{% block icons %}

  <div class="row">
    {% if shortcuts %}

      {% for shortcut in shortcuts %}
        {% if not shortcut.hidden %}
          {% if user.is_authenticated or not shortcut.login_required %}
            <div class="col-sm-3">
              <ul class="nav nav-pills nav-stacked">
                {% if selected_id == shortcut.id %}
                  <li class="active">
                    <a href="{{ shortcut.url }}" class="active">
                {% else %}
                  <li>
                    <a href="{{ shortcut.url }}">
                {% endif %}
                      <center>
                        <img src="{% static 'theme/icons/' %}{{ shortcut.icon }}.png" style="max-width: 100px; height: 100px" />
                        <br>
                        <div class="shortcut-label">
                        {{ shortcut.label|linebreaks }}
                        </div>
                      </center>
                    </a>
                  </li>
              </ul>
            </div>
          {% endif %}
        {% endif %}
      {% endfor %}

    {% else %}

      <h4>
        {% url 'apps' as apps_url %}
        {% blocktrans trimmed %}
          Enable some <a href="{{ apps_url }}">applications</a> to add
          shortcuts to this page.
        {% endblocktrans %}
      </h4>

    {% endif %}
  </div>

  <br>
  {% if details %}
    <div class="panel panel-primary">
      <div class="panel-heading">
        <h3 class="panel-title">{{ details_label }}</h3>
      </div>

      {% for paragraph in details %}
        <div class="panel-body">
          {{ paragraph|safe }}
        </div>
      {% endfor %}
      <br>

      {% if user.is_authenticated and user_is_admin and configure_url %}
        <a class="btn btn-primary btn-sm"
           style="margin-left : 10px; margin-bottom : 5px"
           href="{{ configure_url }}">
          {% trans "Configure &raquo;" %}</a>
      {% endif %}

    </div>
  {% endif %}

{% endblock %}


{% block footer_block %}

  <p>
    <a href="{% url 'index' %}" title="{{ box_name }}">
      <img class="footer-logo" src="{% static 'theme/img/FreedomBox-logo-standard.svg' %}"
           alt="{{ box_name }}" />
    </a>
  </p>

  <p>
    {% blocktrans trimmed %}
      {{ box_name }}, a Debian pure blend, is a 100% free software
      self-hosting web server to deploy social applications on small
      machines. It provides online communication tools respecting your
      privacy and data ownership.
    {% endblocktrans %}
  </p>

  <p>
    {% blocktrans trimmed %}
      This portal is a part of the {{ box_name }} web interface. {{ box_name }}
      is free software, distributed under the GNU Affero General Public License,
      Version 3 or later.
    {% endblocktrans %}
  </p>

  <div class="row">
    <div class="col-md-2 col-md-offset-3">
      <p>
        <a href="https://wiki.debian.org/FreedomBox/Manual">
          {% trans "Manual" %}
        </a>
      </p>
      <p>
        <a href="https://wiki.debian.org/FreedomBox">{% trans "Wiki" %}</a>
      </p>
    </div>
    <div class="col-md-2">
      <p>
        <a href="https://freedombox.org">{% trans "Homepage" %}</a>
      </p>
      <p>
        <a href="https://salsa.debian.org/freedombox-team/">{% trans "Source Code" %}</a>
      </p>
      <p>
        <a href="https://freedomboxfoundation.org/donate/">{% trans "Donate" %}</a>
      </p>
      <p>
        <a href="https://freedomboxfoundation.org/">
          {% trans "FreedomBox Foundation" %}
        </a>
      </p>
    </div>
    <div class="col-md-2">
      <p>
        <a href="https://webchat.oftc.net/?randomnick=1&channels=freedombox&prompt=1">
          {%trans "IRC Chatroom" %}
        </a>
      </p>
      <p>
        <a href="https://lists.alioth.debian.org/mailman/listinfo/freedombox-discuss">
          {% trans "Mailing list" %}
        </a>
      </p>
    </div>
  </div>

  <div class="license-info">
    <p>
      <a href="{% static 'jslicense.html' %}" data-jslicense="1">
        {% trans "JavaScript license information" %}</a>
    </p>
  </div>

{% endblock %}
