{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}
  <h2>{{ title }}</h2>

  {% if expandable_root_size %}
    <p>
      {% blocktrans trimmed %}
        Please backup your data before proceeding.  After this
        operation, {{ expandable_root_size }} of additional free space
        will be available in your root partition.
      {% endblocktrans %}
    </p>
    <form class="form" method="post">
      {% csrf_token %}

      <input type="submit" class="btn btn-primary"
             value="{% trans "Expand Root Partition" %}"/>
    </form>
  {% else %}
    <p>There are no partitions available to expand.
  {% endif %}

{% endblock %}
