{% extends "service.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% block status %}
  {{ block.super }}

  {% if domains %}
    <div class="row">
      <div class="col-sm-6">
        <table class="table table-bordered table-condensed table-striped">
          <thead>
            <tr>
              <th>{% trans "Address" %}</th>
              <th>{% trans "Port" %}</th>
            </tr>
          </thead>
          <tbody>
            {% for domain in domains %}
              <tr>
                <td>
                  {{ domain }}
                </td>
                <td>{{ 30000 }}</td>
              </tr>
            {% endfor %}
          </tbody>
        </table>
      </div>
    </div>
  {% endif %}
{% endblock %}
