{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block page_head %}
  <style type="text/css">
    .wiki-label {
      display: inline-block;
      width: 40%;
    }
    .list-group-item .btn {
      margin: -5px 0;
    }
  </style>
{% endblock %}

{% block content %}

  <div class="row">
    <div class="col-sm-6">
      {% if not sites %}
        <p>{% trans "No wikis or blogs available." %}</p>
        <p>
          <a class="btn btn-primary btn-md"
             href="{% url 'ikiwiki:create' %}">
            {% trans "Create a Wiki or Blog" %}
          </a>
        </p>
      {% else %}
        <div class="list-group">
          {% for site in sites %}
            <div class="list-group-item clearfix">
              <a href="{% url 'ikiwiki:delete' site %}"
                 class="btn btn-default btn-sm pull-right"
                 role="button"
                 title="{% blocktrans %}Delete site {{ site }}{% endblocktrans %}">
                <span class="glyphicon glyphicon-trash"
                      aria-hidden="true"></span>
              </a>

              <a class="wiki-label" href="/ikiwiki/{{ site }}"
                 title="{% blocktrans %}Go to site {{ site }}{% endblocktrans %}">
                {{ site }}
              </a>
            </div>
          {% endfor %}
        </div>
      {% endif %}
    </div>
  </div>

{% endblock %}
