{% extends "base.html" %}
{% comment %}
#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}

  <h2>{{ title }}</h2>

  <p>
    {% blocktrans trimmed %}
      Are you sure you want to shut down?  You will not be able to
      access this web interface after shut down.
    {% endblocktrans %}
  </p>

  {% if pkg_manager_is_busy %}
    <div class="alert alert-danger">
    {% blocktrans trimmed %}
      Currently an installation or upgrade is running.
      Consider waiting until it's finished before shutting down.
    {% endblocktrans %}
    </div>
  {% endif %}

  <form class="form" method="post">
    {% csrf_token %}

    {{ form|bootstrap }}

    {% if pkg_manager_is_busy %}
      <input type="submit" class="btn btn-danger"
             value="{% trans "Shut Down Now" %}"/>
    {% else %}
      <input type="submit" class="btn btn-primary"
             value="{% trans "Shut Down Now" %}"/>
    {% endif %}

  </form>


{% endblock %}
