#
# This file is part of FreedomBox.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Generated by Django 1.9 on 2015-12-04 07:27
#

"""
Initial Django migration for FreedomBox to create database tables.
"""

from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    """Initial migration schema for FreedomBox models."""

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='KVStore',
            fields=[
                ('key', models.TextField(primary_key=True, serialize=False)),
                ('value_json', models.TextField()),
            ],
        ),
    ]
